@@ -3,6 +3,7 @@
 
 import argparse
 import codecs
+import inspect
 import locale
 import logging
 import logging.handlers
@@ -79,7 +80,10 @@ class App(object):
         self.interpreter = None
 
     def _set_streams(self, stdin, stdout, stderr):
-        locale.setlocale(locale.LC_ALL, '')
+        try:
+            locale.setlocale(locale.LC_ALL, '')
+        except locale.Error:
+            pass
         if sys.version_info[:2] == (2, 6):
             # Configure the input and output streams. If a stream is
             # provided, it must be configured correctly by the
@@ -196,6 +200,14 @@ class App(object):
         return
 
     def print_help_if_requested(self):
+        """Print help and exits if deferred help is enabled and requested.
+
+        '--help' shows the help message and exits:
+         * without calling initialize_app if not self.deferred_help (default),
+         * after initialize_app call if self.deferred_help,
+         * during initialize_app call if self.deferred_help and subclass calls
+           explicitly this method in initialize_app.
+        """
         if self.deferred_help and self.options.deferred_help:
             action = HelpAction(None, None, default=self)
             action(self.parser, self.parser, None, None)
@@ -281,7 +293,10 @@ class App(object):
                 self.LOG.error(err)
             return 2
         cmd_factory, cmd_name, sub_argv = subcommand
-        cmd = cmd_factory(self, self.options)
+        kwargs = {}
+        if 'cmd_name' in inspect.getargspec(cmd_factory.__init__).args:
+            kwargs['cmd_name'] = cmd_name
+        cmd = cmd_factory(self, self.options, **kwargs)
         err = None
         result = 1
         try:
